@@ -1,97 +1,101 @@
 #' Discretize numeric variables with XgBoost
 #'
 #' `step_discretize_xgb` creates a *specification* of a recipe step that will
-#'  discretize numeric data (e.g. integers or doubles) into bins in a
-#'  supervised way using an XgBoost model.
+#' discretize numeric data (e.g. integers or doubles) into bins in a supervised
+#' way using an XgBoost model.
 #'
 #' @param recipe A recipe object. The step will be added to the sequence of
 #'   operations for this recipe.
 #' @param ... One or more selector functions to choose which variables are
 #'   affected by the step. See [selections()] for more details.
 #' @param role Defaults to `"predictor"`.
-#' @param trained A logical to indicate if the quantities for preprocessing
-#'   have been estimated.
+#' @param trained A logical to indicate if the quantities for preprocessing have
+#'   been estimated.
 #' @param outcome A call to `vars` to specify which variable is used as the
-#'  outcome to train XgBoost models in order to discretize explanatory
-#'  variables.
-#' @param sample_val Share of data used for validation (with early stopping) of the learned splits
-#' (the rest is used for training). Defaults to 0.20.
+#'   outcome to train XgBoost models in order to discretize explanatory
+#'   variables.
+#' @param sample_val Share of data used for validation (with early stopping) of
+#'   the learned splits (the rest is used for training). Defaults to 0.20.
 #' @param learn_rate The rate at which the boosting algorithm adapts from
-#'  iteration-to-iteration. Corresponds to `eta` in the \pkg{xgboost} package.
-#'  Defaults to 0.3.
+#'   iteration-to-iteration. Corresponds to `eta` in the \pkg{xgboost} package.
+#'   Defaults to 0.3.
 #' @param num_breaks The _maximum_ number of discrete bins to bucket continuous
-#'  features. Corresponds to `max_bin` in the \pkg{xgboost} package. Defaults to
-#'  10.
+#'   features. Corresponds to `max_bin` in the \pkg{xgboost} package. Defaults
+#'   to 10.
 #' @param tree_depth The maximum depth of the tree (i.e. number of splits).
-#' Corresponds to `max_depth` in the \pkg{xgboost} package. Defaults to 1.
+#'   Corresponds to `max_depth` in the \pkg{xgboost} package. Defaults to 1.
 #' @param min_n The minimum number of instances needed to be in each node.
-#' Corresponds to `min_child_weight` in the \pkg{xgboost} package. Defaults to 5.
-#' @param rules The splitting rules of the best XgBoost tree to retain for
-#'  each variable.
+#'   Corresponds to `min_child_weight` in the \pkg{xgboost} package. Defaults to
+#'   5.
+#' @param rules The splitting rules of the best XgBoost tree to retain for each
+#'   variable.
 #' @param id A character string that is unique to this step to identify it.
-#' @param skip A logical. Should the step be skipped when the
-#'  recipe is baked by [recipes::bake()]? While all operations are baked
-#'  when [recipes::prep()] is run, some operations may not be able to be
-#'  conducted on new data (e.g. processing the outcome variable(s)).
-#'  Care should be taken when using `skip = TRUE` as it may affect
-#'  the computations for subsequent operations
+#' @param skip A logical. Should the step be skipped when the recipe is baked by
+#'   [recipes::bake()]? While all operations are baked when [recipes::prep()] is
+#'   run, some operations may not be able to be conducted on new data (e.g.
+#'   processing the outcome variable(s)). Care should be taken when using `skip
+#'   = TRUE` as it may affect the computations for subsequent operations
 #' @template step-return
-#' @export
-#' @details `step_discretize_xgb()` creates non-uniform bins from numerical
-#'  variables by utilizing the information about the outcome variable and
-#'  applying the xgboost model. It is advised to impute missing values before
-#'  this step. This step is intended to be used particularly with linear models
-#'  because thanks to creating non-uniform bins it becomes easier to learn
-#'  non-linear patterns from the data.
+#' @details
 #'
-#'  The best selection of buckets for each variable is selected using
-#'  an internal early stopping scheme implemented in the \pkg{xgboost}
-#'  package, which makes this discretization method prone to overfitting.
+#' `step_discretize_xgb()` creates non-uniform bins from numerical variables by
+#' utilizing the information about the outcome variable and applying the xgboost
+#' model. It is advised to impute missing values before this step. This step is
+#' intended to be used particularly with linear models because thanks to
+#' creating non-uniform bins it becomes easier to learn non-linear patterns from
+#' the data.
 #'
-#' The pre-defined values of the underlying xgboost learns good
-#' and reasonably complex results. However, if one wishes to tune them the
-#' recommended path would be to first start with changing the value of
-#' `num_breaks` to e.g.: 20 or 30. If that doesn't give satisfactory results
-#' one could experiment with modifying the `tree_depth` or `min_n` parameters.
-#' Note that it is not recommended to tune `learn_rate` simultaneously with
-#' other parameters.
+#' The best selection of buckets for each variable is selected using an internal
+#' early stopping scheme implemented in the \pkg{xgboost} package, which makes
+#' this discretization method prone to overfitting.
 #'
-#' This step requires the \pkg{xgboost} package. If not installed, the
-#' step will stop with a note about installing the package.
+#' The pre-defined values of the underlying xgboost learns good and reasonably
+#' complex results. However, if one wishes to tune them the recommended path
+#' would be to first start with changing the value of `num_breaks` to e.g.: 20
+#' or 30. If that doesn't give satisfactory results one could experiment with
+#' modifying the `tree_depth` or `min_n` parameters. Note that it is not
+#' recommended to tune `learn_rate` simultaneously with other parameters.
+#'
+#' This step requires the \pkg{xgboost} package. If not installed, the step will
+#' stop with a note about installing the package.
 #'
 #' Note that the original data will be replaced with the new bins.
-#' 
+#'
 #' # Tidying
 #'
-#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
-#' `terms` (the columns that is selected), `values` is returned.
-#' 
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns `terms`
+#' (the columns that is selected), `values` is returned.
+#'
+#' ```{r, echo = FALSE, results="asis"}
+#' step <- "step_discretize_xgb"
+#' result <- knitr::knit_child("man/rmd/tunable-args.Rmd")
+#' cat(result)
+#' ```
+#'
 #' @template case-weights-supervised
 #'
-#' @examples
-#' library(modeldata)
-#' data(credit_data)
+#' @examplesIf rlang::is_installed(c("xgboost", "modeldata"))
 #' library(rsample)
 #' library(recipes)
+#' data(credit_data, package = "modeldata")
 #'
-#' split <- initial_split(credit_data, strata = "Status")
+#' set.seed(1234)
+#' split <- initial_split(credit_data[1:1000, ], strata = "Status")
 #'
 #' credit_data_tr <- training(split)
 #' credit_data_te <- testing(split)
 #'
 #' xgb_rec <-
-#'   recipe(Status ~ ., data = credit_data_tr) %>%
-#'   step_impute_median(all_numeric()) %>%
-#'   step_discretize_xgb(all_numeric(), outcome = "Status")
+#'   recipe(Status ~ Income + Assets, data = credit_data_tr) %>%
+#'   step_impute_median(Income, Assets) %>%
+#'   step_discretize_xgb(Income, Assets, outcome = "Status")
 #'
-#' if (rlang::is_installed("xgboost")) {
-#'   xgb_rec <- prep(xgb_rec, training = credit_data_tr)
+#' xgb_rec <- prep(xgb_rec, training = credit_data_tr)
 #'
-#'   bake(xgb_rec, credit_data_te, Price)
-#' }
+#' bake(xgb_rec, credit_data_te, Assets)
 #' @seealso [embed::step_discretize_cart()], [recipes::recipe()],
 #' [recipes::prep()], [recipes::bake()]
-
+#' @export
 step_discretize_xgb <-
   function(recipe,
            ...,
@@ -110,12 +114,12 @@ step_discretize_xgb <-
       rlang::abort("`outcome` should select at least one column.")
     }
 
-    recipes::recipes_pkg_check(required_pkgs.step_discretize_xgb())
+    recipes_pkg_check(required_pkgs.step_discretize_xgb())
 
     add_step(
       recipe,
       step_discretize_xgb_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         outcome = outcome,
@@ -153,9 +157,10 @@ step_discretize_xgb_new <-
     )
   }
 
-run_xgboost <- function(.train, .test, .learn_rate, .num_breaks, .tree_depth, .min_n, .objective, .num_class) {
-
-  # Need to set an additional parameter (num_class) when perfoming multi-classification
+run_xgboost <- function(.train, .test, .learn_rate, .num_breaks, .tree_depth,
+                        .min_n, .objective, .num_class) {
+  # Need to set an additional parameter (num_class) when perfoming
+  # multi-classification
   if (.objective == "multi:softprob") {
     .params <- list(
       eta = .learn_rate,
@@ -189,8 +194,9 @@ run_xgboost <- function(.train, .test, .learn_rate, .num_breaks, .tree_depth, .m
   )
 }
 
-xgb_binning <- function(df, outcome, predictor, sample_val, learn_rate, num_breaks, tree_depth, min_n, wts = NULL) {
-
+xgb_binning <- function(df, outcome, predictor, sample_val, learn_rate,
+                        num_breaks, tree_depth, min_n, wts = NULL,
+                        call = caller_env()) {
   # Assuring correct types
   if (is.character(df[[outcome]])) {
     df[[outcome]] <- as.factor(df[[outcome]])
@@ -208,22 +214,26 @@ xgb_binning <- function(df, outcome, predictor, sample_val, learn_rate, num_brea
     df[[outcome]] <- as.integer(df[[outcome]]) - 1
   }
 
-  # Changes: sample_val now is a parameter with 0.20 as default. If sample_val is equal
-  # to 0 then rsample returns it standard error: Error: `prop` must be a number on (0, 1).
-  # If there are less than 2 observations in the test set then an error is given
+  # Changes: sample_val now is a parameter with 0.20 as default. If sample_val
+  # is equal to 0 then rsample returns it standard error: Error: `prop` must be
+  # a number on (0, 1). If there are less than 2 observations in the test set
+  # then an error is given
 
-  # Changes: I also realized that results for a single column are not reproducible given a training set
-  # because sampling is not persistent, therefore I added a specific random seed here
-  # which makes the results much more stable and not so much dependent on inner sampling
+  # Changes: I also realized that results for a single column are not
+  # reproducible given a training set because sampling is not persistent,
+  # therefore I added a specific random seed here which makes the results much
+  # more stable and not so much dependent on inner sampling
   split <- withr::with_seed(
     sample.int(10^6, 1),
     # suppressing rsample messages regarding stratification (regression)
-    suppressWarnings(rsample::initial_split(df, prop = (1 - sample_val), strata = outcome))
+    suppressWarnings(
+      rsample::initial_split(df, prop = (1 - sample_val), strata = outcome)
+    )
   )
 
   train <- rsample::training(split)
   test <- rsample::testing(split)
-  
+
   if (!is.null(wts)) {
     wts <- as.double(wts)
     wts_train <- wts[split$in_id]
@@ -278,7 +288,13 @@ xgb_binning <- function(df, outcome, predictor, sample_val, learn_rate, num_brea
         weight = wts_test
       )
     } else {
-      rlang::abort("Outcome variable doesn't conform to regresion or classification task.")
+      rlang::abort(
+        paste0(
+          "Outcome variable only has less than 2 levels. ",
+          "Doesn't conform to regresion or classification task."
+        ),
+        call = call
+      )
     }
   }
 
@@ -311,9 +327,9 @@ xgb_binning <- function(df, outcome, predictor, sample_val, learn_rate, num_brea
     return(numeric(0))
   }
 
-  # Changes: if there is insufficient training data/ variation then xgboost model is constant
-  # and no splits will be returned. Additional check will inform the user
-  # that the dataset is insufficient for this particular case
+  # Changes: if there is insufficient training data/ variation then xgboost
+  # model is constant and no splits will be returned. Additional check will
+  # inform the user that the dataset is insufficient for this particular case
   # https://github.com/dmlc/xgboost/issues/2876
   # https://stackoverflow.com/questions/42670033/r-getting-non-tree-model-detected-this-function-can-only-be-used-with-tree-mo
   xgb_tree <- try(
@@ -363,18 +379,18 @@ xgb_binning <- function(df, outcome, predictor, sample_val, learn_rate, num_brea
 
 #' @export
 prep.step_discretize_xgb <- function(x, training, info = NULL, ...) {
-  col_names <- recipes::recipes_eval_select(x$terms, training, info)
+  col_names <- recipes_eval_select(x$terms, training, info)
 
-  wts <- recipes::get_case_weights(info, training)
-  were_weights_used <- recipes::are_weights_used(wts)
+  wts <- get_case_weights(info, training)
+  were_weights_used <- are_weights_used(wts)
   if (isFALSE(were_weights_used) || is.null(wts)) {
     wts <- NULL
   }
-  
-  if (length(col_names) > 0) {
-    check_type(training[, col_names])
 
-    y_name <- recipes::recipes_eval_select(x$outcome, training, info)
+  if (length(col_names) > 0) {
+    check_type(training[, col_names], types = c("double", "integer"))
+
+    y_name <- recipes_eval_select(x$outcome, training, info)
 
     col_names <- col_names[col_names != y_name]
 
@@ -390,8 +406,8 @@ prep.step_discretize_xgb <- function(x, training, info = NULL, ...) {
     }
 
     # Changes: check for the minimum number of unique data points in the column
-    # in order to run the step. Otherwise, take it out of col_names. I think that
-    # num_unique = 20 is probably a good default
+    # in order to run the step. Otherwise, take it out of col_names. I think
+    # that num_unique = 20 is probably a good default
     num_unique <- purrr::map_int(training[, col_names], ~ length(unique(.x)))
     too_few <- num_unique < 20
     if (any(too_few)) {
@@ -455,7 +471,7 @@ bake.step_discretize_xgb <- function(object, new_data, ...) {
   vars <- object$rules
 
   check_new_data(names(vars), object, new_data)
-  
+
   for (i in seq_along(vars)) {
     if (length(vars[[i]]) > 0) {
       var <- names(vars)[[i]]
@@ -469,7 +485,7 @@ bake.step_discretize_xgb <- function(object, new_data, ...) {
         dig.lab = 4
       )
 
-      recipes::check_name(binned_data, new_data, object)
+      check_name(binned_data, new_data, object)
       new_data <- binned_data
     }
   }
@@ -477,10 +493,13 @@ bake.step_discretize_xgb <- function(object, new_data, ...) {
 }
 
 #' @export
-print.step_discretize_xgb <- function(x, width = max(20, options()$width - 30), ...) {
+print.step_discretize_xgb <- function(x, width = max(20, options()$width - 30),
+                                      ...) {
   title <- "Discretizing variables using xgboost "
-  print_step(names(x$rules), x$terms, x$trained, title, width,
-             case_weights = x$case_weights)
+  print_step(
+    names(x$rules), x$terms, x$trained, title, width,
+    case_weights = x$case_weights
+  )
   invisible(x)
 }
 
@@ -488,7 +507,7 @@ print.step_discretize_xgb <- function(x, width = max(20, options()$width - 30),
 #' @param x A `step_discretize_xgb` object.
 #' @export
 tidy.step_discretize_xgb <- function(x, ...) {
-  if (recipes::is_trained(x)) {
+  if (is_trained(x)) {
     num_splits <- purrr::map_int(x$rules, length)
 
     res <- tibble(
@@ -496,7 +515,7 @@ tidy.step_discretize_xgb <- function(x, ...) {
       values = unlist(x$rules, use.names = FALSE)
     )
   } else {
-    term_names <- recipes::sel2char(x$terms)
+    term_names <- sel2char(x$terms)
     res <- tibble(
       variable = term_names,
       values = rlang::na_chr
@@ -511,3 +530,21 @@ tidy.step_discretize_xgb <- function(x, ...) {
 required_pkgs.step_discretize_xgb <- function(x, ...) {
   c("xgboost", "embed")
 }
+
+#' @export
+#' @rdname tunable_embed
+tunable.step_discretize_xgb <- function(x, ...) {
+  tibble::tibble(
+    name = c("sample_val", "learn_rate", "num_breaks", "tree_depth", "min_n"),
+    call_info = list(
+      list(pkg = "dials", fun = "validation_set_prop", range = c(0.05, 0.7)),
+      list(pkg = "dials", fun = "learn_rate", range = log10(c(0.05, 0.3))),
+      list(pkg = "dials", fun = "num_breaks", range = c(5, 30)),
+      list(pkg = "dials", fun = "tree_depth"),
+      list(pkg = "dials", fun = "min_n")
+    ),
+    source = "recipe",
+    component = "step_discretize_xgb",
+    component_id = x$id
+  )
+}
