@@ -1,61 +1,74 @@
-#' Supervised and unsupervised uniform manifold approximation and projection (UMAP)
+#' Supervised and unsupervised uniform manifold approximation and projection
+#' (UMAP)
 #'
-#' `step_umap` creates a *specification* of a recipe step that
-#'  will project a set of features into a smaller space.
+#' `step_umap` creates a *specification* of a recipe step that will project a
+#' set of features into a smaller space.
 #'
 #' @inheritParams recipes::step_pca
 #' @param min_dist The effective minimum distance between embedded points.
 #' @param num_comp An integer for the number of UMAP components. If `num_comp`
-#' is greater than the number of selected columns minus one, the smaller value
-#' is used.
+#'   is greater than the number of selected columns minus one, the smaller value
+#'   is used.
 #' @param neighbors An integer for the number of nearest neighbors used to
-#'  construct the target simplicial set. If `neighbors` is greater than the
-#'  number of data points, the smaller value is used.
+#'   construct the target simplicial set. If `neighbors` is greater than the
+#'   number of data points, the smaller value is used.
+#' @param metric Character, type of distance metric to use to find nearest 
+#'   neighbors. See [uwot::umap()] for more details. Default to `"euclidean"`.
 #' @param epochs Number of iterations for the neighbor optimization. See
-#'  [uwot::umap()] for more details.
+#'   [uwot::umap()] for more details.
 #' @param learn_rate Positive number of the learning rate for the optimization
-#'  process.
-#' @param outcome A call to `vars` to specify which variable is
-#'  used as the outcome in the encoding process (if any).
+#'   process.
+#' @param outcome A call to `vars` to specify which variable is used as the
+#'   outcome in the encoding process (if any).
 #' @param options A list of options to pass to [uwot::umap()]. The arguments
-#'  `X`, `n_neighbors`, `n_components`, `min_dist`, `n_epochs`, `ret_model`, and
-#'  `learning_rate` should not be passed here. By default, `verbose` and
-#'  `n_threads` are set.
-#' @param seed Two integers to control the random numbers used by the
-#'  numerical methods. The default pulls from the main session's stream of
-#'  numbers and will give reproducible results if the seed is set prior to
-#'  calling [prep()] or [bake()].
+#'   `X`, `n_neighbors`, `n_components`, `min_dist`, `n_epochs`, `ret_model`,
+#'   and `learning_rate` should not be passed here. By default, `verbose` and
+#'   `n_threads` are set.
+#' @param seed Two integers to control the random numbers used by the numerical
+#'   methods. The default pulls from the main session's stream of numbers and
+#'   will give reproducible results if the seed is set prior to calling [prep()]
+#'   or [bake()].
 #' @param retain Use `keep_original_cols` instead to specify whether the
-#'  original predictors should be retained along with the new embedding
-#'  variables.
-#' @param object An object that defines the encoding. This is
-#'  `NULL` until the step is trained by [recipes::prep()].
+#'   original predictors should be retained along with the new embedding
+#'   variables.
+#' @param object An object that defines the encoding. This is `NULL` until the
+#'   step is trained by [recipes::prep()].
 #' @template step-return
-#' @export
 #' @details
-#' UMAP, short for Uniform Manifold Approximation and Projection, is a nonlinear
-#'  dimension reduction technique that finds local, low-dimensional
-#'  representations of the data. It can be run unsupervised or supervised with
-#'  different types of outcome data (e.g. numeric, factor, etc).
 #'
-#' The new components will have names that begin with `prefix` and a sequence
-#'  of numbers. The variable names are padded with zeros. For example, if
-#'  `num_comp < 10`, their names will be `UMAP1` - `UMAP9`. If `num_comp = 101`,
-#'  the names would be `UMAP001` - `UMAP101`.
-#'  
+#' UMAP, short for Uniform Manifold Approximation and Projection, is a nonlinear
+#' dimension reduction technique that finds local, low-dimensional
+#' representations of the data. It can be run unsupervised or supervised with
+#' different types of outcome data (e.g. numeric, factor, etc).
+#'
+#' The new components will have names that begin with `prefix` and a sequence of
+#' numbers. The variable names are padded with zeros. For example, if `num_comp
+#' < 10`, their names will be `UMAP1` - `UMAP9`. If `num_comp = 101`, the names
+#' would be `UMAP001` - `UMAP101`.
+#'
 #' # Tidying
 #'
-#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
-#' `terms` (the selectors or variables selected) is returned.
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns `terms`
+#' (the selectors or variables selected) is returned.
 #' 
+#' ```{r, echo = FALSE, results="asis"}
+#' step <- "step_umap"
+#' result <- knitr::knit_child("man/rmd/tunable-args.Rmd")
+#' cat(result)
+#' ```
+#'
 #' @template case-weights-not-supported
 #'
+#' @template bundle
+#'
 #' @references
-#' McInnes, L., & Healy, J. (2018). UMAP: Uniform Manifold Approximation and
-#'  Projection for Dimension Reduction. \url{ https://arxiv.org/abs/1802.03426}.
 #'
-#' "How UMAP Works" \url{https://umap-learn.readthedocs.io/en/latest/how_umap_works.html}
+#' McInnes, L., & Healy, J. (2018). UMAP: Uniform Manifold
+#' Approximation and Projection for Dimension Reduction. 
+#' \url{https://arxiv.org/abs/1802.03426}.
 #'
+#' "How UMAP Works"
+#' \url{https://umap-learn.readthedocs.io/en/latest/how_umap_works.html}
 #'
 #' @examples
 #' library(recipes)
@@ -78,6 +91,7 @@
 #' bake(supervised, new_data = te, Species, starts_with("umap")) %>%
 #'   ggplot(aes(x = UMAP1, y = UMAP2, col = Species)) +
 #'   geom_point(alpha = .5)
+#' @export
 step_umap <-
   function(recipe,
            ...,
@@ -87,6 +101,7 @@ step_umap <-
            neighbors = 15,
            num_comp = 2,
            min_dist = 0.01,
+           metric = "euclidean",
            learn_rate = 1,
            epochs = NULL,
            options = list(verbose = FALSE, n_threads = 1),
@@ -106,8 +121,8 @@ step_umap <-
       keep_original_cols <- retain
     }
 
-    recipes::recipes_pkg_check(required_pkgs.step_umap())
-    if (is.numeric(seed) & !is.integer(seed)) {
+    recipes_pkg_check(required_pkgs.step_umap())
+    if (is.numeric(seed) && !is.integer(seed)) {
       seed <- as.integer(seed)
     }
     if (length(seed) != 2) {
@@ -117,13 +132,14 @@ step_umap <-
     add_step(
       recipe,
       step_umap_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         outcome = outcome,
         neighbors = neighbors,
         num_comp = num_comp,
         min_dist = min_dist,
+        metric = metric,
         learn_rate = learn_rate,
         epochs = epochs,
         options = options,
@@ -139,7 +155,7 @@ step_umap <-
   }
 
 step_umap_new <-
-  function(terms, role, trained, outcome, neighbors, num_comp, min_dist,
+  function(terms, role, trained, outcome, neighbors, num_comp, min_dist, metric,
            learn_rate, epochs, options, seed, prefix, keep_original_cols,
            retain, object, skip, id) {
     step(
@@ -151,6 +167,7 @@ step_umap_new <-
       neighbors = neighbors,
       num_comp = num_comp,
       min_dist = min_dist,
+      metric = metric,
       learn_rate = learn_rate,
       epochs = epochs,
       options = options,
@@ -165,7 +182,11 @@ step_umap_new <-
   }
 
 umap_fit_call <- function(obj, y = NULL) {
-  cl <- rlang::call2("umap", .ns = "uwot", X = rlang::expr(training[, col_names]))
+  cl <- rlang::call2(
+    "umap",
+    .ns = "uwot",
+    X = rlang::expr(training[, col_names])
+  )
   if (!is.null(y)) {
     cl$y <- rlang::expr(training[[y_name]])
   }
@@ -174,6 +195,7 @@ umap_fit_call <- function(obj, y = NULL) {
   cl$n_epochs <- obj$epochs
   cl$learning_rate <- obj$learn_rate
   cl$min_dist <- obj$min_dist
+  cl$metric <- obj$metric
   if (length(obj$options) > 0) {
     cl <- rlang::call_modify(cl, !!!obj$options)
   }
@@ -181,15 +203,13 @@ umap_fit_call <- function(obj, y = NULL) {
   cl
 }
 
-
-
 #' @export
 prep.step_umap <- function(x, training, info = NULL, ...) {
-  col_names <- recipes::recipes_eval_select(x$terms, training, info)
+  col_names <- recipes_eval_select(x$terms, training, info)
 
   if (length(col_names) > 0) {
     if (length(x$outcome) > 0) {
-      y_name <- recipes::recipes_eval_select(x$outcome, training, info)
+      y_name <- recipes_eval_select(x$outcome, training, info)
     } else {
       y_name <- NULL
     }
@@ -213,6 +233,7 @@ prep.step_umap <- function(x, training, info = NULL, ...) {
     neighbors = x$neighbors,
     num_comp = x$num_comp,
     min_dist = x$min_dist,
+    metric = x$metric,
     learn_rate = x$learn_rate,
     epochs = x$epochs,
     options = x$options,
@@ -229,7 +250,7 @@ prep.step_umap <- function(x, training, info = NULL, ...) {
 #' @export
 bake.step_umap <- function(object, new_data, ...) {
   check_new_data(names(object$object$xnames), object, new_data)
-  
+
   if (length(object$object) == 0) {
     return(new_data)
   }
@@ -245,12 +266,16 @@ bake.step_umap <- function(object, new_data, ...) {
 
   if (is.null(object$prefix)) object$prefix <- "UMAP"
 
-  res <- recipes::check_name(res, new_data, object)
-  new_data <- bind_cols(new_data, as_tibble(res))
+  colnames(res) <- names0(object$num_comp, prefix = object$prefix)
+  res <- as_tibble(res)
+  
+  res <- check_name(res, new_data, object, names(res))
+  new_data <- bind_cols(new_data, res)
 
-  keep_original_cols <- recipes::get_keep_original_cols(object)
+  keep_original_cols <- get_keep_original_cols(object)
   if (!keep_original_cols) {
-    new_data <- new_data[, !(colnames(new_data) %in% object$object$xnames), drop = FALSE]
+    keep_cols <- !(colnames(new_data) %in% object$object$xnames)
+    new_data <- new_data[, keep_cols, drop = FALSE]
   }
 
   new_data
@@ -278,9 +303,26 @@ tidy.step_umap <- function(x, ...) {
   res
 }
 
-
 #' @rdname required_pkgs.embed
 #' @export
 required_pkgs.step_umap <- function(x, ...) {
   c("uwot", "embed")
 }
+
+#' @export
+#' @rdname tunable_embed
+tunable.step_umap <- function(x, ...) {
+  tibble::tibble(
+    name = c("num_comp", "neighbors", "min_dist", "learn_rate", "epochs"),
+    call_info = list(
+      list(pkg = "dials", fun = "num_comp", range = c(1, 10)),
+      list(pkg = "dials", fun = "neighbors", range = c(5, 200)),
+      list(pkg = "dials", fun = "min_dist", range = c(-4, -0.69897)),
+      list(pkg = "dials", fun = "learn_rate"),
+      list(pkg = "dials", fun = "epochs", range = c(100, 700))
+    ),
+    source = "recipe",
+    component = "step_umap",
+    component_id = x$id
+  )
+}
